@@ -15,7 +15,7 @@ module Hledger.Cli.CliOptions (
   hiddenflags,
   inputflags,
   reportflags,
-  outputflags,
+  -- outputflags,
   outputFormatFlag,
   outputFileFlag,
   generalflagsgroup1,
@@ -35,6 +35,7 @@ module Hledger.Cli.CliOptions (
   CliOpts(..),
   defcliopts,
   getHledgerCliOpts,
+  getHledgerCliOpts',
   rawOptsToCliOpts,
   checkCliOpts,
   outputFormats,
@@ -73,8 +74,10 @@ import qualified Control.Exception as C
 import Control.Monad (when)
 import Data.Char
 import Data.Default
+import Data.Either (isRight)
 import Data.Functor.Identity (Identity)
 import "base-compat-batteries" Data.List.Compat
+import Data.List.Extra (nubSort)
 import Data.List.Split (splitOneOf)
 import Data.Ord
 import Data.Maybe
@@ -121,7 +124,6 @@ inputflags :: [Flag RawOpts]
 inputflags = [
   flagReq  ["file","f"]      (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use - (default: $LEDGER_FILE or $HOME/.hledger.journal)"
  ,flagReq  ["rules-file"]    (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
- ,flagReq  ["separator"]     (\s opts -> Right $ setopt "separator" s opts) "SEP" "CSV separator (default: ,)"
  ,flagReq  ["alias"]         (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "rename accounts named OLD to NEW"
  ,flagNone ["anon"]          (setboolopt "anon") "anonymize accounts and payees"
  ,flagReq  ["pivot"]         (\s opts -> Right $ setopt "pivot" s opts)  "TAGNAME" "use some other field/tag for account names"
@@ -153,7 +155,7 @@ reportflags = [
 
   -- valuation
  ,flagNone ["B","cost"]      (setboolopt "B")
-   "show amounts converted to their cost, using the transaction price. Equivalent to --value=cost."
+   "show amounts converted to their cost/selling amount, using the transaction price. Equivalent to --value=cost."
  ,flagNone ["V","market"]    (setboolopt "V")
    (unwords
      ["show amounts converted to current market value (single period reports)"
@@ -176,11 +178,16 @@ reportflags = [
      ,"- current market value, in default valuation commodity or COMM"
      ,"- market value on the given date, in default valuation commodity or COMM"
      ])
+ ,flagNone ["infer-value"]   (setboolopt "infer-value") "with -V/-X/--value, also infer market prices from transactions"
 
   -- generated postings/transactions
  ,flagNone ["auto"]          (setboolopt "auto") "apply automated posting rules to modify transactions"
- ,flagNone ["forecast"]      (setboolopt "forecast") "apply periodic transaction rules to generate future transactions, to 6 months from now or report end date"
-
+ ,flagOpt "" ["forecast"]    (\s opts -> Right $ setopt "forecast" s opts) "PERIODEXP" 
+  (unlines
+   [ "Generate periodic transactions (from periodic transaction rules). By default these begin after the latest recorded transaction, and end 6 months from today, or at the report end date."
+   , "Also, in hledger-ui, make future transactions visible."
+   , "Note that = (and not a space) is required before PERIODEXP if you wish to supply it."
+   ])
  ]
 
 -- | Common flags that are accepted but not shown in --help,
@@ -191,9 +198,18 @@ hiddenflags = [
  ]
 
 -- | Common output-related flags: --output-file, --output-format...
-outputflags = [outputFormatFlag, outputFileFlag]
-outputFormatFlag = flagReq  ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT" "select the output format. Supported formats:\ntxt, csv, html."
-outputFileFlag   = flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE" "write output to FILE. A file extension matching one of the above formats selects that format."
+
+-- outputflags = [outputFormatFlag, outputFileFlag]
+
+outputFormatFlag :: [String] -> Flag RawOpts
+outputFormatFlag fmts = flagReq
+  ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT"
+  ("select the output format. Supported formats:\n" ++ intercalate ", " fmts ++ ".")
+
+outputFileFlag :: Flag RawOpts
+outputFileFlag = flagReq
+  ["output-file","o"] (\s opts -> Right $ setopt "output-file" s opts) "FILE"
+  "write output to FILE. A file extension matching one of the above formats selects that format."
 
 argsFlag :: FlagHelp -> Arg RawOpts
 argsFlag desc = flagArg (\s opts -> Right $ setopt "args" s opts) desc
@@ -469,9 +485,8 @@ checkCliOpts opts =
 -- Empty lines in the pre/postamble are removed by cmdargs;
 -- add a space character to preserve them.
 --
-getHledgerCliOpts :: Mode RawOpts -> IO CliOpts
-getHledgerCliOpts mode' = do
-  args' <- getArgs >>= expandArgsAt
+getHledgerCliOpts' :: Mode RawOpts -> [String] -> IO CliOpts
+getHledgerCliOpts' mode' args' = do
   let rawopts = either usageError id $ process mode' args'
   opts <- rawOptsToCliOpts rawopts
   debugArgs args' opts
@@ -498,6 +513,11 @@ getHledgerCliOpts mode' = do
         d <- getCurrentDay
         putStrLn $ "search query: " ++ show (queryFromOpts d $ reportopts_ opts)
 
+getHledgerCliOpts :: Mode RawOpts -> IO CliOpts
+getHledgerCliOpts mode' = do
+  args' <- getArgs >>= expandArgsAt
+  getHledgerCliOpts' mode' args' 
+
 -- CliOpts accessors
 
 -- | Get the (tilde-expanded, absolute) journal file path from
@@ -648,8 +668,8 @@ hledgerAddons = do
 stripPrognamePrefix = drop (length progname + 1)
 
 dropRedundantSourceVersion [f,g]
-  | takeExtension f `elem` compiledExts = [f]
-  | takeExtension g `elem` compiledExts = [g]
+  | map toLower (takeExtension f) `elem` compiledExts = [f]
+  | map toLower (takeExtension g) `elem` compiledExts = [g]
 dropRedundantSourceVersion fs = fs
 
 compiledExts = ["",".com",".exe"]
@@ -661,7 +681,7 @@ likelyExecutablesInPath :: IO [String]
 likelyExecutablesInPath = do
   pathdirs <- splitOneOf "[:;]" `fmap` getEnvSafe "PATH"
   pathfiles <- concat `fmap` mapM getDirectoryContentsSafe pathdirs
-  return $ nub $ sort pathfiles
+  return $ nubSort pathfiles
   -- exclude directories and files without execute permission.
   -- These will do a stat for each hledger-*, probably ok.
   -- But they need paths, not just filenames
