@@ -43,7 +43,7 @@ accountsmode = hledgerCommandMode
   ,flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "flat mode: omit N leading account name parts"
   ]
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY]")
 
 -- | The accounts command.
@@ -66,24 +66,24 @@ accounts CliOpts{rawopts_=rawopts, reportopts_=ropts} j = do
       accts                = dbg5 "accts to show" $ -- no need to nub/sort, accountTree will
         if | declared     && not used -> matcheddeclaredaccts
            | not declared && used     -> matchedusedaccts
-           | otherwise                -> matcheddeclaredaccts ++ matchedusedaccts 
+           | otherwise                -> matcheddeclaredaccts ++ matchedusedaccts
 
   -- 2. sort them by declaration order and name, at each level of their tree structure
       sortedaccts = sortAccountNamesByDeclaration j tree accts
 
-  -- 3. if there's a depth limit, depth-clip and remove any no longer useful items 
+  -- 3. if there's a depth limit, depth-clip and remove any no longer useful items
       clippedaccts =
         dbg1 "clippedaccts" $
         filter (matchesAccount acctq) $  -- clipping can leave accounts that no longer match the query, remove such
         nub $                          -- clipping can leave duplicates (adjacent, hopefully)
         filter (not . T.null) $        -- depth:0 can leave nulls
-        map (clipAccountName depth) $  -- clip at depth if specified 
-        sortedaccts 
+        map (clipAccountName depth) $  -- clip at depth if specified
+        sortedaccts
 
-  -- 4. print what remains as a list or tree, maybe applying --drop in the former case 
+  -- 4. print what remains as a list or tree, maybe applying --drop in the former case
   mapM_ (T.putStrLn . render) clippedaccts
     where
-      render a 
+      render a
         | tree_ ropts = T.replicate (2 * (accountNameLevel a - 1)) " " <> accountLeafName a
         | otherwise   = accountNameDrop (drop_ ropts) a
 
