@@ -293,8 +293,9 @@ balancemode = hledgerCommandMode
    ,flagNone ["invert"] (setboolopt "invert") "display all amounts with reversed sign"
    ,flagNone ["transpose"] (setboolopt "transpose") "transpose rows and columns"
    ,flagNone ["percent", "%"] (setboolopt "percent") "express values in percentage of each column's total"
+   ,outputFormatFlag ["txt","html","csv","json"]
+   ,outputFileFlag
    ]
-   ++ outputflags
   )
   [generalflagsgroup1]
   hiddenflags
@@ -309,26 +310,28 @@ balance opts@CliOpts{rawopts_=rawopts,reportopts_=ropts@ReportOpts{..}} j = do
     Right _ -> do
       let budget      = boolopt "budget" rawopts
           multiperiod = interval_ /= NoInterval
-          format      = outputFormatFromOpts opts
+          fmt         = outputFormatFromOpts opts
 
       if budget then do  -- single or multi period budget report
         reportspan <- reportSpan j ropts
         let budgetreport     = dbg1 "budgetreport"     $ budgetReport ropts assrt reportspan d j
               where
                 assrt          = not $ ignore_assertions_ $ inputopts_ opts
-            render = case format of
-              "csv"  -> const $ error' "Sorry, CSV output is not yet implemented for this kind of report."  -- TODO
-              "html" -> const $ error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
-              _      -> budgetReportAsText ropts
+            render = case fmt of
+              "txt"  -> budgetReportAsText ropts
+              "json" -> (++"\n") . TL.unpack . toJsonText
+              _      -> const $ error' $ unsupportedOutputFormatError fmt
         writeOutput opts $ render budgetreport
 
       else
         if multiperiod then do  -- multi period balance report
-          let report = multiBalanceReport ropts (queryFromOpts d ropts) j
-              render = case format of
-                "csv"  -> (++ "\n") . printCSV . multiBalanceReportAsCsv ropts
-                "html" ->  (++ "\n") . TL.unpack . L.renderText . multiBalanceReportAsHtml ropts
-                _      -> multiBalanceReportAsText ropts
+          let report = multiBalanceReport d ropts j
+              render = case fmt of
+                "txt"  -> multiBalanceReportAsText ropts
+                "csv"  -> (++"\n") . printCSV . multiBalanceReportAsCsv ropts
+                "html" -> (++"\n") . TL.unpack . L.renderText . multiBalanceReportAsHtml ropts
+                "json" -> (++"\n") . TL.unpack . toJsonText
+                _      -> const $ error' $ unsupportedOutputFormatError fmt
           writeOutput opts $ render report
 
         else do  -- single period simple balance report
@@ -339,10 +342,11 @@ balance opts@CliOpts{rawopts_=rawopts,reportopts_=ropts@ReportOpts{..}} j = do
                     in balanceReportFromMultiBalanceReport ropts' (queryFromOpts d ropts) j
                           -- for historical balances we must use balanceReportFromMultiBalanceReport (also forces --no-elide)
                 | otherwise = balanceReport ropts (queryFromOpts d ropts) j -- simple Ledger-style balance report
-              render = case format of
+              render = case fmt of
+                "txt"  -> balanceReportAsText
                 "csv"  -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
-                "html" -> \_ _ -> error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
-                _      -> balanceReportAsText
+                "json" -> const $ (++"\n") . TL.unpack . toJsonText
+                _      -> const $ error' $ unsupportedOutputFormatError fmt
           writeOutput opts $ render ropts report
 
 -- rendering single-column balance reports
@@ -458,7 +462,8 @@ renderComponent1 opts (acctname, depth, total) (FormatField ljust min max field)
 -- The CSV will always include the initial headings row,
 -- and will include the final totals row unless --no-total is set.
 multiBalanceReportAsCsv :: ReportOpts -> MultiBalanceReport -> CSV
-multiBalanceReportAsCsv opts@ReportOpts{average_, row_total_} (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+multiBalanceReportAsCsv opts@ReportOpts{average_, row_total_}
+    (PeriodicReport colspans items (PeriodicReportRow _ _ coltotals tot avg)) =
   maybetranspose $
   ("Account" : map showDateSpan colspans
    ++ ["Total"   | row_total_]
@@ -469,7 +474,7 @@ multiBalanceReportAsCsv opts@ReportOpts{average_, row_total_} (MultiBalanceRepor
    (amts
     ++ [rowtot | row_total_]
     ++ [rowavg | average_])
-  | (a, _, _, amts, rowtot, rowavg) <- items]
+  | PeriodicReportRow a _ amts rowtot rowavg <- items]
   ++
   if no_total_ opts
   then []
@@ -583,9 +588,10 @@ multiBalanceReportAsText ropts@ReportOpts{..} r =
         PeriodChange       -> "Balance changes"
         CumulativeChange   -> "Ending balances (cumulative)"
         HistoricalBalance  -> "Ending balances (historical)")
-      (showDateSpan $ multiBalanceReportSpan r)
+      (showDateSpan $ periodicReportSpan r)
       (case value_ of
         Just (AtCost _mc)   -> ", valued at cost"
+        Just (AtThen _mc)   -> error' unsupportedValueThenError  -- TODO -- ", valued at period ends"  -- handled like AtEnd for now
         Just (AtEnd _mc)    -> ", valued at period ends"
         Just (AtNow _mc)    -> ", current value"
         -- XXX duplicates the above
@@ -596,7 +602,8 @@ multiBalanceReportAsText ropts@ReportOpts{..} r =
 
 -- | Build a 'Table' from a multi-column balance report.
 balanceReportAsTable :: ReportOpts -> MultiBalanceReport -> Table String String MixedAmount
-balanceReportAsTable opts@ReportOpts{average_, row_total_, balancetype_} (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+balanceReportAsTable opts@ReportOpts{average_, row_total_, balancetype_}
+    (PeriodicReport colspans items (PeriodicReportRow _ _ coltotals tot avg)) =
    maybetranspose $
    addtotalrow $
    Table
@@ -604,7 +611,7 @@ balanceReportAsTable opts@ReportOpts{average_, row_total_, balancetype_} (MultiB
      (T.Group NoLine $ map Header colheadings)
      (map rowvals items)
   where
-    totalscolumn = row_total_ && not (balancetype_ `elem` [CumulativeChange, HistoricalBalance])
+    totalscolumn = row_total_ && balancetype_ `notElem` [CumulativeChange, HistoricalBalance]
     mkDate = case balancetype_ of
        PeriodChange -> showDateSpanMonthAbbrev
        _            -> maybe "" (showDate . prevday) . spanEnd
@@ -612,10 +619,10 @@ balanceReportAsTable opts@ReportOpts{average_, row_total_, balancetype_} (MultiB
                   ++ ["  Total" | totalscolumn]
                   ++ ["Average" | average_]
     accts = map renderacct items
-    renderacct (a,a',i,_,_,_)
-      | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
+    renderacct (PeriodicReportRow a i _ _ _)
+      | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack (accountLeafName a)
       | otherwise  = T.unpack $ maybeAccountNameDrop opts a
-    rowvals (_,_,_,as,rowtot,rowavg) = as
+    rowvals (PeriodicReportRow _ _ as rowtot rowavg) = as
                              ++ [rowtot | totalscolumn]
                              ++ [rowavg | average_]
     addtotalrow | no_total_ opts = id
