@@ -10,7 +10,6 @@ module Hledger.Cli.Commands.Incomestatement (
  ,incomestatement
 ) where
 
-import Data.String.Here
 import System.Console.CmdArgs.Explicit
 
 import Hledger
@@ -18,18 +17,7 @@ import Hledger.Cli.CliOptions
 import Hledger.Cli.CompoundBalanceCommand
 
 incomestatementSpec = CompoundBalanceCommandSpec {
-  cbcname     = "incomestatement",
-  cbcaliases  = ["is"],
-  cbchelp     = [here|
-This command displays a simple income statement, showing revenues
-and expenses during a period. It assumes that these accounts are under a 
-top-level `revenue` or `income` or `expense` account (case insensitive,
-plural forms also allowed).
-
-Note this report shows all account balances with normal positive sign
-(like conventional financial statements, unlike balance/print/register)
-(experimental).
-  |],
+  cbcdoc      = $(embedFileRelative "Hledger/Cli/Commands/Incomestatement.txt"),
   cbctitle    = "Income Statement",
   cbcqueries  = [
      CBCSubreportSpec{
