@@ -5,7 +5,7 @@
 module Hledger.Cli.Commands.Rewrite (
   rewritemode
  ,rewrite
-) 
+)
 where
 
 #if !(MIN_VERSION_base(4,11,0))
@@ -29,14 +29,14 @@ rewritemode = hledgerCommandMode
   ,flagNone ["diff"] (setboolopt "diff") "generate diff suitable as an input for patch tool"
   ]
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY] --add-posting \"ACCT  AMTEXPR\" ...")
 
 -- TODO regex matching and interpolating matched name in replacement
 -- TODO interpolating match groups in replacement
 -- TODO allow using this on unbalanced entries, eg to rewrite while editing
 
-rewrite opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j@Journal{jtxns=ts} = do 
+rewrite opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j@Journal{jtxns=ts} = do
   -- rewrite matched transactions
   let modifiers = transactionModifierFromOpts opts : jtxnmodifiers j
   let j' = j{jtxns=modifyTransactions modifiers ts}
@@ -46,11 +46,11 @@ rewrite opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j@Journal{jtxns=ts} = d
 -- | Build a 'TransactionModifier' from any query arguments and --add-posting flags
 -- provided on the command line, or throw a parse error.
 transactionModifierFromOpts :: CliOpts -> TransactionModifier
-transactionModifierFromOpts CliOpts{rawopts_=rawopts,reportopts_=ropts} = 
+transactionModifierFromOpts CliOpts{rawopts_=rawopts,reportopts_=ropts} =
   TransactionModifier{tmquerytxt=q, tmpostingrules=ps}
   where
     q = T.pack $ query_ ropts
-    ps = map (parseposting . stripquotes . T.pack) $ listofstringopt "add-posting" rawopts
+    ps = map (parseposting . T.pack) $ listofstringopt "add-posting" rawopts
     parseposting t = either (error' . errorBundlePretty) id ep
       where
         ep = runIdentity (runJournalParser (postingp Nothing <* eof) t')
@@ -132,7 +132,7 @@ diffTxn j t t' =
                 diffs = map mapDiff $ D.getDiff source changed'
                 source | Just contents <- lookup fp $ jfiles j = map T.unpack . drop (line-1) . take line' $ T.lines contents
                        | otherwise = []
-                changed = lines $ showTransactionUnelided t'
+                changed = lines $ showTransaction t'
                 changed' | null changed = changed
                          | null $ last changed = init changed
                          | otherwise = changed
