@@ -4,32 +4,38 @@
 module Hledger.Cli.Commands.Tags (
   tagsmode
  ,tags
-) 
+)
 where
 
-import Data.List
+import Data.List.Extra (nubSort)
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
 import Safe
+import System.Console.CmdArgs.Explicit as C
 import Hledger
 import Hledger.Cli.CliOptions
 
 tagsmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Tags.txt")
-  [] -- [flagNone ["strict"] (setboolopt "strict") "makes date comparing strict"] -- 
+  [flagNone ["values"] (setboolopt "values") "list tag values instead of tag names"
+  ]
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[TAGREGEX [QUERY...]]")
 
 tags CliOpts{rawopts_=rawopts,reportopts_=ropts} j = do
   d <- getCurrentDay
   let
     args      = listofstringopt "args" rawopts
-    mtagpats  = headMay args
+    mtagpat   = headMay args
     queryargs = drop 1 args
-    q = queryFromOpts d $ ropts{query_ = unwords queryargs} 
+    values    = boolopt "values" rawopts
+    q = queryFromOpts d $ ropts{query_ = unwords $ map quoteIfNeeded queryargs}
     txns = filter (q `matchesTransaction`) $ jtxns $ journalSelectingAmountFromOpts ropts j
-    tags = 
-      nub $ sort $ 
-      (maybe id (filter . regexMatchesCI) mtagpats) $ 
-      map (T.unpack . fst) $ concatMap transactionAllTags txns
-  mapM_ putStrLn tags
+    tagsorvalues =
+      nubSort $
+      [if values then v else t
+      | (t,v) <- concatMap transactionAllTags txns
+      , maybe True (`regexMatchesCI` T.unpack t) mtagpat
+      ]
+  mapM_ T.putStrLn tagsorvalues
