@@ -18,7 +18,6 @@ import Data.Maybe
 import qualified Data.Text as TS
 import qualified Data.Text.Lazy as TL
 import Data.Time.Calendar
-import Data.Time.Format
 import System.Console.CmdArgs.Explicit as C
 import Hledger.Read.CsvReader (CSV, printCSV)
 import Lucid as L hiding (value_)
@@ -27,7 +26,7 @@ import Text.Tabular as T
 import Hledger
 import Hledger.Cli.Commands.Balance
 import Hledger.Cli.CliOptions
-import Hledger.Cli.Utils (writeOutput)
+import Hledger.Cli.Utils (unsupportedOutputFormatError, writeOutput)
 
 -- | Description of a compound balance report command,
 -- from which we generate the command's cmdargs mode and IO action.
@@ -107,7 +106,7 @@ compoundBalanceCommandMode CompoundBalanceCommandSpec{..} =
     ,flagNone ["pretty-tables"] (setboolopt "pretty-tables") "use unicode when displaying tables"
     ,flagNone ["sort-amount","S"] (setboolopt "sort-amount") "sort by amount instead of account code/name"
     ,flagNone ["percent", "%"] (setboolopt "percent") "express values in percentage of each column's total"
-    ,outputFormatFlag
+    ,outputFormatFlag ["txt","html","csv","json"]
     ,outputFileFlag
     ]
     [generalflagsgroup1]
@@ -143,23 +142,23 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
         no_total_=if percent_ && length cbcqueries > 1 then True else no_total_
       }
       userq = queryFromOpts d ropts'
-      format = outputFormatFromOpts opts
+      fmt = outputFormatFromOpts opts
 
       -- make a CompoundBalanceReport.
       -- For efficiency, generate a price oracle here and reuse it with each subreport.
-      priceoracle = journalPriceOracle j
+      priceoracle = journalPriceOracle infer_value_ j
       subreports =
         map (\CBCSubreportSpec{..} ->
                 (cbcsubreporttitle
-                ,mbrNormaliseSign cbcsubreportnormalsign $ -- <- convert normal-negative to normal-positive
+                ,prNormaliseSign cbcsubreportnormalsign $ -- <- convert normal-negative to normal-positive
                   compoundBalanceSubreport ropts' userq j priceoracle cbcsubreportquery cbcsubreportnormalsign
                 ,cbcsubreportincreasestotal
                 ))
             cbcqueries
 
       subtotalrows =
-        [(coltotals, increasesoveralltotal)
-        | (_, MultiBalanceReport (_,_,(coltotals,_,_)), increasesoveralltotal) <- subreports
+        [(prrAmounts $ prTotals report, increasesoveralltotal)
+        | (_, report, increasesoveralltotal) <- subreports
         ]
 
       -- Sum the subreport totals by column. Handle these cases:
@@ -186,7 +185,7 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
 
       colspans =
         case subreports of
-          (_, MultiBalanceReport (ds,_,_), _):_ -> ds
+          (_, PeriodicReport ds _ _, _):_ -> ds
           [] -> []
 
       title =
@@ -218,6 +217,7 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
 
           valuationdesc = case value_ of
             Just (AtCost _mc)   -> ", valued at cost"
+            Just (AtThen _mc)   -> error' unsupportedValueThenError  -- TODO
             Just (AtEnd _mc)    -> ", valued at period ends"
             Just (AtNow _mc)    -> ", current value"
             Just (AtDefault _mc) | multiperiod   -> ", valued at period ends"
@@ -235,10 +235,12 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
 
     -- render appropriately
     writeOutput opts $
-      case format of
+      case fmt of
+        "txt"  -> compoundBalanceReportAsText ropts' cbr
         "csv"  -> printCSV (compoundBalanceReportAsCsv ropts cbr) ++ "\n"
-        "html" -> (++ "\n") $ TL.unpack $ L.renderText $ compoundBalanceReportAsHtml ropts cbr
-        _      -> compoundBalanceReportAsText ropts' cbr
+        "html" -> (++"\n") $ TL.unpack $ L.renderText $ compoundBalanceReportAsHtml ropts cbr
+        "json" -> (++"\n") $ TL.unpack $ toJsonText cbr
+        _      -> error' $ unsupportedOutputFormatError fmt
 
 -- | Summarise one or more (inclusive) end dates, in a way that's
 -- visually different from showDateSpan, suggesting discrete end dates
@@ -246,11 +248,11 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
 showEndDates :: [Day] -> String
 showEndDates es = case es of
   -- cf showPeriod
-  (e:_:_) -> showdate e ++ ",," ++ showdate (last es)
+  (e:_:_) -> showdate e ++ ".." ++ showdate (last es)
   [e]     -> showdate e
   []      -> ""
   where
-    showdate = formatTime defaultTimeLocale "%0C%y/%m/%d" 
+    showdate = show
 
 -- | Run one subreport for a compound balance command in multi-column mode.
 -- This returns a MultiBalanceReport.
@@ -261,20 +263,20 @@ compoundBalanceSubreport ropts@ReportOpts{..} userq j priceoracle subreportqfn s
     ropts' = ropts { empty_=True, normalbalance_=Just subreportnormalsign }
     -- run the report
     q = And [subreportqfn j, userq]
-    r@(MultiBalanceReport (dates, rows, totals)) = multiBalanceReportWith ropts' q j priceoracle
+    r@(PeriodicReport dates rows totals) = multiBalanceReportWith ropts' q j priceoracle
     -- if user didn't specify --empty, now remove the all-zero rows, unless they have non-zero subaccounts
     -- in this report
     r' | empty_    = r
-       | otherwise = MultiBalanceReport (dates, rows', totals)
+       | otherwise = PeriodicReport dates rows' totals
           where
             nonzeroaccounts =
-              dbg1 "nonzeroaccounts" $
-              catMaybes $ map (\(act,_,_,amts,_,_) ->
-                            if not (all isZeroMixedAmount amts) then Just act else Nothing) rows
+              dbg5 "nonzeroaccounts" $
+              mapMaybe (\(PeriodicReportRow act _ amts _ _) ->
+                            if not (all mixedAmountLooksZero amts) then Just act else Nothing) rows
             rows' = filter (not . emptyRow) rows
               where
-                emptyRow (act,_,_,amts,_,_) =
-                  all isZeroMixedAmount amts && all (not . (act `isAccountNamePrefixOf`)) nonzeroaccounts
+                emptyRow (PeriodicReportRow act _ amts _ _) =
+                  all mixedAmountLooksZero amts && not (any (act `isAccountNamePrefixOf`) nonzeroaccounts)
 
 -- | Render a compound balance report as plain text suitable for console output.
 {- Eg:
@@ -363,12 +365,11 @@ compoundBalanceReportAsCsv ropts (title, colspans, subreports, (coltotals, grand
         numcols
           | null subreports = 1
           | otherwise =
-            (3 +) $ -- account name & indent columns
+            (1 +) $ -- account name column
             (if row_total_ ropts then (1+) else id) $
             (if average_ ropts then (1+) else id) $
             maximum $ -- depends on non-null subreports
-            map (\(MultiBalanceReport (amtcolheadings, _, _)) -> length amtcolheadings) $
-            map second3 subreports
+            map (length . prDates . second3) subreports
     addtotals
       | no_total_ ropts || length subreports == 1 = id
       | otherwise = (++
