@@ -10,6 +10,7 @@ Hledger.Utils.
 
 module Hledger.Cli.Utils
     (
+     unsupportedOutputFormatError,
      withJournalDo,
      writeOutput,
      journalTransform,
@@ -57,6 +58,10 @@ import Hledger.Read
 import Hledger.Reports
 import Hledger.Utils
 
+-- | Standard error message for a bad output format specified with -O/-o.
+unsupportedOutputFormatError :: String -> String
+unsupportedOutputFormatError fmt = "Sorry, output format \""++fmt++"\" is unrecognised or not yet implemented for this report or report mode."
+
 -- | Parse the user's specified journal file(s) as a Journal, maybe apply some
 -- transformations according to options, and run a hledger command with it.
 -- Or, throw an error.
@@ -109,18 +114,22 @@ anonymiseByOpts opts =
 -- from today if unspecified.
 --
 journalAddForecast :: CliOpts -> Journal -> IO Journal
-journalAddForecast opts@CliOpts{inputopts_=iopts, reportopts_=ropts} j = do
+journalAddForecast CliOpts{inputopts_=iopts, reportopts_=ropts} j = do
   today <- getCurrentDay
 
   -- "They can start no earlier than: the day following the latest normal transaction in the journal (or today if there are none)."
   let mjournalend   = dbg2 "journalEndDate" $ journalEndDate False j  -- ignore secondary dates
-      forecaststart = dbg2 "forecaststart" $ fromMaybe today mjournalend
+      forecastbeginDefault = dbg2 "forecastbeginDefault" $ fromMaybe today mjournalend
 
   -- "They end on or before the specified report end date, or 180 days from today if unspecified."
   mspecifiedend <-  snd . dbg2 "specifieddates" <$> specifiedStartEndDates ropts
-  let forecastend = dbg2 "forecastend" $ fromMaybe (addDays 180 today) mspecifiedend
-
-  let forecastspan = DateSpan (Just forecaststart) (Just forecastend)
+  let forecastendDefault = dbg2 "forecastendDefault" $ fromMaybe (addDays 180 today) mspecifiedend
+      
+  let forecastspan = dbg2 "forecastspan" $
+        spanDefaultsFrom
+          (fromMaybe nulldatespan $ dbg2 "forecastspan flag" $ forecast_ ropts)
+          (DateSpan (Just forecastbeginDefault) (Just forecastendDefault))
+          
       forecasttxns =
         [ txnTieKnot t | pt <- jperiodictxns j
                        , t <- runPeriodicTransaction pt forecastspan
@@ -130,12 +139,12 @@ journalAddForecast opts@CliOpts{inputopts_=iopts, reportopts_=ropts} j = do
       forecasttxns' = (if auto_ iopts then modifyTransactions (jtxnmodifiers j) else id) forecasttxns
 
   return $
-    if forecast_ ropts
-      then journalBalanceTransactions' opts j{ jtxns = concat [jtxns j, forecasttxns'] }
-      else j
+    case forecast_ ropts of
+      Just _  -> journalBalanceTransactions' iopts j{ jtxns = concat [jtxns j, forecasttxns'] }
+      Nothing -> j
   where
-    journalBalanceTransactions' opts j =
-      let assrt = not . ignore_assertions_ $ inputopts_ opts
+    journalBalanceTransactions' iopts j =
+      let assrt = not . ignore_assertions_ $ iopts
       in
        either error' id $ journalBalanceTransactions assrt j
 
@@ -230,6 +239,14 @@ openBrowserOn u = trybrowsers browsers u
 -- overwrite it with this new text, or give an error, but only if the text
 -- is different from the current file contents, and return a flag
 -- indicating whether we did anything.
+--
+-- The given text should have unix line endings (\n); the existing
+-- file content will be normalised to unix line endings before
+-- comparing the two. If the file is overwritten, the new file will
+-- have the current system's native line endings (\n on unix, \r\n on
+-- windows). This could be different from the file's previous line
+-- endings, if working with a DOS file on unix or vice-versa.
+--
 writeFileWithBackupIfChanged :: FilePath -> T.Text -> IO Bool
 writeFileWithBackupIfChanged f t = do
   s <- readFilePortably f
